@@ -20,12 +20,14 @@
 
     You can also give an explicit caption, e.g. :exmpl:`Foo <foo>`.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from six import iteritems
 from docutils import nodes, utils
 
+import sphinx
 from sphinx.util.nodes import split_explicit_title
 
 
@@ -36,10 +38,10 @@ def make_link_role(base_url, prefix):
         try:
             full_url = base_url % part
         except (TypeError, ValueError):
-            env = inliner.document.settings.env
-            env.warn(env.docname, 'unable to expand %s extlink with base '
-                     'URL %r, please make sure the base contains \'%%s\' '
-                     'exactly once' % (typ, base_url))
+            inliner.reporter.warning(
+                'unable to expand %s extlink with base URL %r, please make '
+                'sure the base contains \'%%s\' exactly once'
+                % (typ, base_url), line=lineno)
             full_url = base_url + part
         if not has_explicit_title:
             if prefix is None:
@@ -50,10 +52,13 @@ def make_link_role(base_url, prefix):
         return [pnode], []
     return role
 
+
 def setup_link_roles(app):
-    for name, (base_url, prefix) in app.config.extlinks.iteritems():
+    for name, (base_url, prefix) in iteritems(app.config.extlinks):
         app.add_role(name, make_link_role(base_url, prefix))
 
+
 def setup(app):
     app.add_config_value('extlinks', {}, 'env')
     app.connect('builder-inited', setup_link_roles)
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
