@@ -5,40 +5,43 @@
 
     Handlers for additional ReST roles.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
-import warnings
 
+from six import iteritems
 from docutils import nodes, utils
 from docutils.parsers.rst import roles
 
 from sphinx import addnodes
 from sphinx.locale import _
+from sphinx.errors import SphinxError
 from sphinx.util import ws_re
-from sphinx.util.nodes import split_explicit_title
+from sphinx.util.nodes import split_explicit_title, process_index_entry, \
+    set_role_source_info
 
 
 generic_docroles = {
-    'command' : nodes.strong,
-    'dfn' : nodes.emphasis,
-    'kbd' : nodes.literal,
-    'mailheader' : addnodes.literal_emphasis,
-    'makevar' : nodes.strong,
-    'manpage' : addnodes.literal_emphasis,
-    'mimetype' : addnodes.literal_emphasis,
-    'newsgroup' : addnodes.literal_emphasis,
-    'program' : nodes.strong,  # XXX should be an x-ref
-    'regexp' : nodes.literal,
+    'command': addnodes.literal_strong,
+    'dfn': nodes.emphasis,
+    'kbd': nodes.literal,
+    'mailheader': addnodes.literal_emphasis,
+    'makevar': addnodes.literal_strong,
+    'manpage': addnodes.manpage,
+    'mimetype': addnodes.literal_emphasis,
+    'newsgroup': addnodes.literal_emphasis,
+    'program': addnodes.literal_strong,  # XXX should be an x-ref
+    'regexp': nodes.literal,
 }
 
-for rolename, nodeclass in generic_docroles.iteritems():
+for rolename, nodeclass in iteritems(generic_docroles):
     generic = roles.GenericRole(rolename, nodeclass)
     role = roles.CustomRole(rolename, generic, {'classes': [rolename]})
     roles.register_local_role(rolename, role)
 
+
 # -- generic cross-reference role ----------------------------------------------
 
 class XRefRole(object):
@@ -95,7 +98,11 @@ class XRefRole(object):
                  options={}, content=[]):
         env = inliner.document.settings.env
         if not typ:
-            typ = env.config.default_role
+            typ = env.temp_data.get('default_role')
+            if not typ:
+                typ = env.config.default_role
+            if not typ:
+                raise SphinxError('cannot determine default role!')
         else:
             typ = typ.lower()
         if ':' not in typ:
@@ -126,7 +133,7 @@ class XRefRole(object):
         refnode = self.nodeclass(rawtext, reftype=role, refdomain=domain,
                                  refexplicit=has_explicit_title)
         # we may need the line number for warnings
-        refnode.line = lineno
+        set_role_source_info(inliner, lineno, refnode)
         title, target = self.process_link(
             env, refnode, has_explicit_title, title, target)
         # now that the target and title are finally determined, set them
@@ -141,16 +148,15 @@ class XRefRole(object):
     # methods that can be overwritten
 
     def process_link(self, env, refnode, has_explicit_title, title, target):
-        """
-        Called after parsing title and target text, and creating the reference
-        node (given in *refnode*).  This method can alter the reference node and
-        must return a new (or the same) ``(title, target)`` tuple.
+        """Called after parsing title and target text, and creating the
+        reference node (given in *refnode*).  This method can alter the
+        reference node and must return a new (or the same) ``(title, target)``
+        tuple.
         """
         return title, ws_re.sub(' ', target)
 
     def result_nodes(self, document, env, node, is_ref):
-        """
-        Called before returning the finished nodes.  *node* is the reference
+        """Called before returning the finished nodes.  *node* is the reference
         node if one was created (*is_ref* is then true), else the content node.
         This method can add other nodes and must return a ``(nodes, messages)``
         tuple (the usual return value of a role function).
@@ -158,7 +164,16 @@ class XRefRole(object):
         return [node], []
 
 
-def indexmarkup_role(typ, rawtext, etext, lineno, inliner,
+class AnyXRefRole(XRefRole):
+    def process_link(self, env, refnode, has_explicit_title, title, target):
+        result = XRefRole.process_link(self, env, refnode, has_explicit_title,
+                                       title, target)
+        # add all possible context info (i.e. std:program, py:module etc.)
+        refnode.attributes.update(env.ref_context)
+        return result
+
+
+def indexmarkup_role(typ, rawtext, text, lineno, inliner,
                      options={}, content=[]):
     """Role for PEP/RFC references that generate an index entry."""
     env = inliner.document.settings.env
@@ -166,48 +181,54 @@ def indexmarkup_role(typ, rawtext, etext, lineno, inliner,
         typ = env.config.default_role
     else:
         typ = typ.lower()
-    text = utils.unescape(etext)
+    has_explicit_title, title, target = split_explicit_title(text)
+    title = utils.unescape(title)
+    target = utils.unescape(target)
     targetid = 'index-%s' % env.new_serialno('index')
     indexnode = addnodes.index()
     targetnode = nodes.target('', '', ids=[targetid])
     inliner.document.note_explicit_target(targetnode)
     if typ == 'pep':
         indexnode['entries'] = [
-            ('single', _('Python Enhancement Proposals!PEP %s') % text,
-             targetid, 'PEP %s' % text)]
+            ('single', _('Python Enhancement Proposals; PEP %s') % target,
+             targetid, '', None)]
         anchor = ''
-        anchorindex = text.find('#')
+        anchorindex = target.find('#')
         if anchorindex > 0:
-            text, anchor = text[:anchorindex], text[anchorindex:]
+            target, anchor = target[:anchorindex], target[anchorindex:]
+        if not has_explicit_title:
+            title = "PEP " + utils.unescape(title)
         try:
-            pepnum = int(text)
+            pepnum = int(target)
         except ValueError:
-            msg = inliner.reporter.error('invalid PEP number %s' % text,
+            msg = inliner.reporter.error('invalid PEP number %s' % target,
                                          line=lineno)
             prb = inliner.problematic(rawtext, rawtext, msg)
             return [prb], [msg]
         ref = inliner.document.settings.pep_base_url + 'pep-%04d' % pepnum
-        sn = nodes.strong('PEP '+text, 'PEP '+text)
+        sn = nodes.strong(title, title)
         rn = nodes.reference('', '', internal=False, refuri=ref+anchor,
                              classes=[typ])
         rn += sn
         return [indexnode, targetnode, rn], []
     elif typ == 'rfc':
-        indexnode['entries'] = [('single', 'RFC; RFC %s' % text,
-                                 targetid, 'RFC %s' % text)]
+        indexnode['entries'] = [
+            ('single', 'RFC; RFC %s' % target, targetid, '', None)]
         anchor = ''
-        anchorindex = text.find('#')
+        anchorindex = target.find('#')
         if anchorindex > 0:
-            text, anchor = text[:anchorindex], text[anchorindex:]
+            target, anchor = target[:anchorindex], target[anchorindex:]
+        if not has_explicit_title:
+            title = "RFC " + utils.unescape(title)
         try:
-            rfcnum = int(text)
+            rfcnum = int(target)
         except ValueError:
-            msg = inliner.reporter.error('invalid RFC number %s' % text,
+            msg = inliner.reporter.error('invalid RFC number %s' % target,
                                          line=lineno)
             prb = inliner.problematic(rawtext, rawtext, msg)
             return [prb], [msg]
         ref = inliner.document.settings.rfc_base_url + inliner.rfc_url % rfcnum
-        sn = nodes.strong('RFC '+text, 'RFC '+text)
+        sn = nodes.strong(title, title)
         rn = nodes.reference('', '', internal=False, refuri=ref+anchor,
                              classes=[typ])
         rn += sn
@@ -216,13 +237,14 @@ def indexmarkup_role(typ, rawtext, etext, lineno, inliner,
 
 _amp_re = re.compile(r'(?<!&)&(?![&\s])')
 
+
 def menusel_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
     text = utils.unescape(text)
     if typ == 'menuselection':
         text = text.replace('-->', u'\N{TRIANGULAR BULLET}')
     spans = _amp_re.split(text)
 
-    node = nodes.emphasis(rawtext=rawtext)
+    node = nodes.inline(rawtext=rawtext)
     for i, span in enumerate(spans):
         span = span.replace('&&', '&')
         if i == 0:
@@ -241,8 +263,10 @@ def menusel_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
     node['classes'].append(typ)
     return [node], []
 
+
 _litvar_re = re.compile('{([^}]+)}')
 
+
 def emph_literal_role(typ, rawtext, text, lineno, inliner,
                       options={}, content=[]):
     text = utils.unescape(text)
@@ -259,23 +283,56 @@ def emph_literal_role(typ, rawtext, text, lineno, inliner,
     return [retnode], []
 
 
-_abbr_re = re.compile('\((.*)\)$')
+_abbr_re = re.compile('\((.*)\)$', re.S)
+
 
 def abbr_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
     text = utils.unescape(text)
     m = _abbr_re.search(text)
     if m is None:
-        return [addnodes.abbreviation(text, text)], []
+        return [addnodes.abbreviation(text, text, **options)], []
     abbr = text[:m.start()].strip()
     expl = m.group(1)
-    return [addnodes.abbreviation(abbr, abbr, explanation=expl)], []
+    options = options.copy()
+    options['explanation'] = expl
+    return [addnodes.abbreviation(abbr, abbr, **options)], []
+
+
+def index_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
+    # create new reference target
+    env = inliner.document.settings.env
+    targetid = 'index-%s' % env.new_serialno('index')
+    targetnode = nodes.target('', '', ids=[targetid])
+    # split text and target in role content
+    has_explicit_title, title, target = split_explicit_title(text)
+    title = utils.unescape(title)
+    target = utils.unescape(target)
+    # if an explicit target is given, we can process it as a full entry
+    if has_explicit_title:
+        entries = process_index_entry(target, targetid)
+    # otherwise we just create a "single" entry
+    else:
+        # but allow giving main entry
+        main = ''
+        if target.startswith('!'):
+            target = target[1:]
+            title = title[1:]
+            main = 'main'
+        entries = [('single', target, targetid, main, None)]
+    indexnode = addnodes.index()
+    indexnode['entries'] = entries
+    set_role_source_info(inliner, lineno, indexnode)
+    textnode = nodes.Text(title, title)
+    return [indexnode, targetnode, textnode], []
 
 
 specific_docroles = {
     # links to download references
     'download': XRefRole(nodeclass=addnodes.download_reference),
     # links to documents
-    'doc': XRefRole(),
+    'doc': XRefRole(warn_dangling=True, innernodeclass=nodes.inline),
+    # links to anything
+    'any': AnyXRefRole(warn_dangling=True),
 
     'pep': indexmarkup_role,
     'rfc': indexmarkup_role,
@@ -284,14 +341,8 @@ specific_docroles = {
     'file': emph_literal_role,
     'samp': emph_literal_role,
     'abbr': abbr_role,
+    'index': index_role,
 }
 
-for rolename, func in specific_docroles.iteritems():
+for rolename, func in iteritems(specific_docroles):
     roles.register_local_role(rolename, func)
-
-
-# backwards compatibility alias
-def xfileref_role(*args, **kwds):
-    warnings.warn('xfileref_role is deprecated, use XRefRole',
-                  DeprecationWarning, stacklevel=2)
-    return XRefRole()(*args, **kwds)
