@@ -5,20 +5,23 @@
 
     Stuff for docutils compatibility.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+import warnings
 
 from docutils import nodes
+from docutils.parsers.rst import Directive  # noqa
+
+from docutils import __version__ as _du_version
+docutils_version = tuple(int(x) for x in _du_version.split('.')[:2])
+
 
-# function missing in docutils 0.5
 def make_admonition(node_class, name, arguments, options, content, lineno,
                     content_offset, block_text, state, state_machine):
-    #if not content:
-    #    error = state_machine.reporter.error(
-    #        'The "%s" admonition is empty; content required.' % (name),
-    #        nodes.literal_block(block_text, block_text), line=lineno)
-    #    return [error]
+    warnings.warn('make_admonition is deprecated, use '
+                  'docutils.parsers.rst.directives.admonitions.BaseAdmonition '
+                  'instead', DeprecationWarning, stacklevel=2)
     text = '\n'.join(content)
     admonition_node = node_class(text)
     if arguments:
@@ -33,15 +36,3 @@ def make_admonition(node_class, name, arguments, options, content, lineno,
         admonition_node['classes'] += classes
     state.nested_parse(content, content_offset, admonition_node)
     return [admonition_node]
-
-
-# backwards-compatibility aliases for helpers in older Sphinx versions that
-# supported the docutils 0.4 directive function interface
-
-from docutils.parsers.rst import Directive
-
-def directive_dwim(obj):
-    import warnings
-    warnings.warn('directive_dwim is deprecated and no longer needed',
-                  DeprecationWarning, stacklevel=2)
-    return obj
